#include <cstdio>
#include <cstdlib>

#include "../include/chip8.h"
#include "../include/instruction.h"
#include "../include/instruction_handler.h"
#include "../include/display.h"

Chip8::Chip8() : instructionHandler(), display() {
    this->pc = 0x200;
}

Chip8::~Chip8() {
    delete display;
    delete instructionHandler;
}

void Chip8::loadRom(const std::string& file) {
    FILE* game = fopen(("roms/" + file).c_str(), "rb");
    if (game == nullptr) {

        exit(EXIT_FAILURE);
    }

    fread(this->memory + 0x200, sizeof(char), 0xFFF - 0x200, game);
    fclose(game);
}

Display* Chip8::getDisplay() const {
    return this->display;
}

void Chip8::emulateCycle() {
    unsigned short opcode = (this->memory[this->pc] << 8 | this->memory[this->pc + 1]);
    this->instructionHandler->handleInstruction(opcode & 0xF000, *this, Instruction(opcode));
}
